package com.objis.spring.dao.hibernate.test;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.core.io.ClassPathResource;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.orm.hibernate3.HibernateTemplate;

import com.objis.spring.dao.IEmployeDao;
import com.objis.spring.dao.hibernate.HibernateDaoSupportEmployeDao;
import com.objis.spring.dao.hibernate.HibernateEmployeDao;
//import com.objis.spring.dao.hibernate.HibernateNonIntrusiveEmployeDao;

import domaine.Employe;

public class DemoHIBERNATE {
	
	public static void main(String[] args) {
		Logger logger = Logger.getLogger(DemoHIBERNATE.class);

logger.fatal("Connexion  une base de donnes avec Spring et Enregistrement d'un Employe");

///Cration Employe Sans spcifier l'ID (c'est Hibernate qui propose implmentation ID)

Employe emp = new Employe("douglas","objis","Douglas","MBIANDOU","douglas.mbiandou@objis.com","manager");

logger.info("Le Nom de l'employe  enregistrer est " + emp.getNom());
logger.debug("Le Prnom de l'employe  enregistrer est " + emp.getPrenom());

/*
 * Enregistrement d'un Employe via Hibernate 
 * 1) Spring HibernateTemplate (Fait) : saveEmployeHibernateTemplate
 * 2) Spring Non intrusif Hibernate DAO (Fait) : saveEmployeNonIntrusiveHibernateTemplate
 * 3) Spring Hibernate DAO Support : saveEmployeHibernateDaoSupport
 **/

//saveEmployeHibernateTemplate(emp);
saveEmployeHibernateDAO(emp);
//saveEmployeHibernateNonIntrusiveDAO(emp);

	}

	/*
	 * Utilisation typique d'un Template Hibernate avec Spring ( via Template HibernateTemplate ). 
	 * 
	 */

public static void saveEmployeHibernateTemplate(Employe employe) {
		ApplicationContext appContext = new ClassPathXmlApplicationContext("spring-data.xml");
		HibernateTemplate hibernateTemplate =  (HibernateTemplate)appContext.getBean("hibernateTemplate");
		hibernateTemplate.save(employe);
	}
	
	/*
	 * Utilisation d'un typique DAO Spring ( via Template HibernateTemplate ). 
	 * Obligatoire avec hibernate 2 
	 * Utile avec Hibernate 3 si peu de DAO dans l'application. Sinon prfrer version DaoSupport pour que 
	 * tous les DAOs partagent une seule HibernateTemplate. 
	 */
	
	public static void saveEmployeHibernateDAO(Employe employe) {
		
		ApplicationContext appContext = new ClassPathXmlApplicationContext("spring-data.xml");
		IEmployeDao hibernateEmployeDao = (IEmployeDao) appContext.getBean("hibernateEmployeDao");
		hibernateEmployeDao.saveEmploye(employe);
	}
	
	/*
	 * Utilisation d'un DAO non intrusif ( utilisation de la Session contextuelle )
	 * AVANTAGE : dcouplage code DAO / code Spring
	 * INCONVENIENT : Exceptions spcifiques Hibernates leves. 
	 * -->Manque de transparence sur framework de mapping utilis.
	 * Solution : intercepter les leves d'exceptions hibernate via aspect 
	 */
	public static void saveEmployeHibernateNonIntrusiveDAO(Employe employe) {
		
		ApplicationContext appContext = new ClassPathXmlApplicationContext("spring-data.xml");
		//MAUVAISE PRATIQUE : passer par une classe concrte. Risque d'erreur : java.lang.ClassCastException: $Proxy9 cannot be cast to ....
		//HibernateNonIntrusiveEmployeDao hibernateNonIntrusiveEmployeDao = (HibernateNonIntrusiveEmployeDao) appContext.getBean("hibernateNonIntrusiveEmployeDao");
		//BONNE PRATIQUE : passer par interface 
		IEmployeDao hibernateNonIntrusiveEmployeDao = (IEmployeDao) appContext.getBean("hibernateNonIntrusiveEmployeDao");
		hibernateNonIntrusiveEmployeDao.saveEmploye(employe);
		//Remarque : Afin d'appliquer comportement transactionnel, 
		//Spring cre par dfaut un proxy dynamique implmentant la mme interface que votre bean
	}

}
